@@ -303,6 +303,31 @@ soup_dns_ntop (struct sockaddr *sa)
 	}
 }
 
+gboolean
+soup_dns_is_ip_address (const char *name)
+{
+	struct sockaddr_in sin;
+#ifdef HAVE_IPV6
+	struct sockaddr_in6 sin6;
+
+	if (inet_pton (AF_INET, name, &sin.sin_addr) > 0 ||
+	    inet_pton (AF_INET6, name, &sin6.sin6_addr) > 0)
+		return TRUE;
+#else /* !HAVE_IPV6 */
+#if defined(HAVE_INET_PTON)
+	if (inet_pton (AF_INET, name, &sin.sin_addr) > 0)
+		return TRUE;
+#elif defined(HAVE_INET_ATON)
+	if (inet_aton (name, &sin.sin_addr) != 0)
+		return TRUE;
+#else
+	if (inet_addr (name) != INADDR_NONE)
+		return TRUE;
+#endif
+#endif /* HAVE_IPV6 */
+	return FALSE;
+}
+
 static void
 resolve_address (SoupDNSCacheEntry *entry)
 {
@@ -312,7 +337,7 @@ resolve_address (SoupDNSCacheEntry *entry)
 	int retval;
 
 	memset (&hints, 0, sizeof (struct addrinfo));
-#  ifdef HAVE_AI_ADDRCONFIG
+#  ifdef AI_ADDRCONFIG
 	hints.ai_flags = AI_CANONNAME | AI_ADDRCONFIG;
 #  else
 	hints.ai_flags = AI_CANONNAME;
@@ -326,8 +351,7 @@ resolve_address (SoupDNSCacheEntry *entry)
 		entry->sockaddr = g_memdup (res->ai_addr, res->ai_addrlen);
 		entry->resolved = TRUE;
 		freeaddrinfo (res);
-	} else
-		entry->resolved = (retval != EAI_AGAIN);
+	}
 
 #else /* !HAVE_GETADDRINFO */
 
@@ -343,9 +367,7 @@ resolve_address (SoupDNSCacheEntry *entry)
 		memcpy (&sin.sin_addr, h->h_addr_list[0], sizeof (struct in_addr));
 		entry->sockaddr = g_memdup (&sin, sizeof (struct sockaddr_in));
 		entry->resolved = TRUE;
-	} else
-		entry->resolved = (h || h_errno != TRY_AGAIN);
-
+	}
 
 	g_mutex_unlock (soup_gethost_lock);
 
@@ -375,10 +397,8 @@ resolve_name (SoupDNSCacheEntry *entry)
 	if (retval == 0) {
 		entry->hostname = name;
 		entry->resolved = TRUE;
-	} else {
+	} else
 		g_free (name);
-		entry->resolved = (retval != EAI_AGAIN);
-	}
 
 #else /* !HAVE_GETNAMEINFO */
 
@@ -392,8 +412,7 @@ resolve_name (SoupDNSCacheEntry *entry)
 		if (h) {
 			entry->hostname = g_strdup (h->h_name);
 			entry->resolved = TRUE;
-		} else
-			entry->resolved = (h_errno != TRY_AGAIN);
+		}
 	}
 
 	g_mutex_unlock (soup_gethost_lock);
@@ -471,7 +490,7 @@ soup_dns_lookup_address (struct sockaddr *sockaddr)
 
 	entry = soup_dns_cache_entry_lookup (name);
 	if (!entry)
-		entry = soup_dns_cache_entry_new (name); // FIXME
+		entry = soup_dns_cache_entry_new (name); /* FIXME */
 	g_free (name);
 
 	lookup = g_slice_new0 (SoupDNSLookup);
@@ -534,7 +553,7 @@ resolver_thread (gpointer user_data)
 		lookup = async_lookups->data;
 		async_lookups = g_slist_remove (async_lookups, lookup);
 
-		soup_add_idle (lookup->async_context, do_async_callback, lookup);
+		soup_add_completion (lookup->async_context, do_async_callback, lookup);
 	}
 
 	soup_dns_cache_entry_unref (entry);
@@ -614,7 +633,7 @@ async_cancel (GCancellable *cancellable, gpointer user_data)
 	if (g_slist_find (entry->async_lookups, lookup)) {
 		entry->async_lookups = g_slist_remove (entry->async_lookups,
 						       lookup);
-		soup_add_idle (lookup->async_context, do_async_callback, lookup);
+		soup_add_completion (lookup->async_context, do_async_callback, lookup);
 	}
 
 	g_mutex_unlock (soup_dns_lock);
@@ -661,7 +680,7 @@ soup_dns_lookup_resolve_async (SoupDNSLookup *lookup,
 						 FALSE, NULL);
 		}
 	} else
-		soup_add_idle (lookup->async_context, do_async_callback, lookup);
+		soup_add_completion (lookup->async_context, do_async_callback, lookup);
 
 	g_mutex_unlock (soup_dns_lock);
 }
